/**
  * This file is part of the KDE project
  * Copyright (C) 2009 Kevin Ottens <ervin@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */

#ifndef QTJOLIE_ABSTRACTADAPTOR_H
#define QTJOLIE_ABSTRACTADAPTOR_H

#include <QtCore/QObject>

namespace Jolie
{
class AbstractAdaptorPrivate;
class Server;
class ServerPrivate;
class Message;

class Q_DECL_EXPORT AbstractAdaptor : public QObject
{
    Q_OBJECT
protected:
    AbstractAdaptor(QObject *parent = 0);
public:
    ~AbstractAdaptor();


private:
    virtual void relay(Server *server, int clientId, const Message &message) = 0;

    friend class ServerPrivate;

    AbstractAdaptorPrivate * const d;
};

} // namespace Jolie

#endif

