// This file has been automatically generated by "generate_keys.sh" 
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public License
// along with this library; see the file COPYING.LIB.  If not, write to
// the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,

#include <QtCore/QCOORD> 
#include "kckey.h" 

const KKeys kde_KKEYS[NB_KEYS] = {
	{ "Escape", 0x1000 }, // translated
	{ "Tab", 0x1001 }, // translated
	{ "Backtab", 0x1002 }, // translated
	{ "Backspace", 0x1003 }, // translated
	{ "Return", 0x1004 }, // translated
	{ "Enter", 0x1005 }, // translated
	{ "Insert", 0x1006 }, // translated
	{ "Delete", 0x1007 }, // translated
	{ "Pause", 0x1008 }, // translated
	{ "Print", 0x1009 }, // translated
	{ "SysReq", 0x100a }, // translated
	{ "Clear", 0x100b },
	{ "Home", 0x1010 }, // translated
	{ "End", 0x1011 }, // translated
	{ "Left", 0x1012 }, // translated
	{ "Up", 0x1013 }, // translated
	{ "Right", 0x1014 }, // translated
	{ "Down", 0x1015 }, // translated
	{ "Prior", 0x1016 }, // translated
	{ "Next", 0x1017 }, // translated
	{ "Shift", 0x1020 }, // translated
	{ "Control", 0x1021 }, // translated
	{ "Meta", 0x1022 }, // translated
	{ "Alt", 0x1023 }, // translated
	{ "CapsLock", 0x1024 }, // translated
	{ "NumLock", 0x1025 }, // translated
	{ "ScrollLock", 0x1026 }, // translated
	{ "F1", 0x1030 },
	{ "F2", 0x1031 },
	{ "F3", 0x1032 },
	{ "F4", 0x1033 },
	{ "F5", 0x1034 },
	{ "F6", 0x1035 },
	{ "F7", 0x1036 },
	{ "F8", 0x1037 },
	{ "F9", 0x1038 },
	{ "F10", 0x1039 },
	{ "F11", 0x103a },
	{ "F12", 0x103b },
	{ "F13", 0x103c },
	{ "F14", 0x103d },
	{ "F15", 0x103e },
	{ "F16", 0x103f },
	{ "F17", 0x1040 },
	{ "F18", 0x1041 },
	{ "F19", 0x1042 },
	{ "F20", 0x1043 },
	{ "F21", 0x1044 },
	{ "F22", 0x1045 },
	{ "F23", 0x1046 },
	{ "F24", 0x1047 },
	{ "F25", 0x1048 },
	{ "F26", 0x1049 },
	{ "F27", 0x104a },
	{ "F28", 0x104b },
	{ "F29", 0x104c },
	{ "F30", 0x104d },
	{ "F31", 0x104e },
	{ "F32", 0x104f },
	{ "F33", 0x1050 },
	{ "F34", 0x1051 },
	{ "F35", 0x1052 },
	{ "Super_L", 0x1053 },
	{ "Super_R", 0x1054 },
	{ "Menu", 0x1055 }, // translated
	{ "Hyper_L", 0x1056 },
	{ "Hyper_R", 0x1057 },
	{ "Help", 0x1058 }, // translated
	{ "Direction_L", 0x1059 },
	{ "Direction_R", 0x1060 },
	{ "Space", 0x20 }, // translated
	{ "Any", Qt::Key_Space },
	{ "Exclam", 0x21 }, // translated
	{ "QuoteDbl", 0x22 },
	{ "NumberSign", 0x23 }, // translated
	{ "Dollar", 0x24 }, // translated
	{ "Percent", 0x25 }, // translated
	{ "Ampersand", 0x26 }, // translated
	{ "Apostrophe", 0x27 }, // translated
	{ "ParenLeft", 0x28 }, // translated
	{ "ParenRight", 0x29 }, // translated
	{ "Asterisk", 0x2a }, // translated
	{ "Plus", 0x2b }, // translated
	{ "Comma", 0x2c }, // translated
	{ "Minus", 0x2d }, // translated
	{ "Period", 0x2e }, // translated
	{ "Slash", 0x2f }, // translated
	{ "0", 0x30 },
	{ "1", 0x31 },
	{ "2", 0x32 },
	{ "3", 0x33 },
	{ "4", 0x34 },
	{ "5", 0x35 },
	{ "6", 0x36 },
	{ "7", 0x37 },
	{ "8", 0x38 },
	{ "9", 0x39 },
	{ "Colon", 0x3a }, // translated
	{ "Semicolon", 0x3b }, // translated
	{ "Less", 0x3c }, // translated
	{ "Equal", 0x3d }, // translated
	{ "Greater", 0x3e }, // translated
	{ "Question", 0x3f }, // translated
	{ "At", 0x40 },
	{ "A", 0x41 },
	{ "B", 0x42 },
	{ "C", 0x43 },
	{ "D", 0x44 },
	{ "E", 0x45 },
	{ "F", 0x46 },
	{ "G", 0x47 },
	{ "H", 0x48 },
	{ "I", 0x49 },
	{ "J", 0x4a },
	{ "K", 0x4b },
	{ "L", 0x4c },
	{ "M", 0x4d },
	{ "N", 0x4e },
	{ "O", 0x4f },
	{ "P", 0x50 },
	{ "Q", 0x51 },
	{ "R", 0x52 },
	{ "S", 0x53 },
	{ "T", 0x54 },
	{ "U", 0x55 },
	{ "V", 0x56 },
	{ "W", 0x57 },
	{ "X", 0x58 },
	{ "Y", 0x59 },
	{ "Z", 0x5a },
	{ "BracketLeft", 0x5b }, // translated
	{ "Backslash", 0x5c }, // translated
	{ "BracketRight", 0x5d }, // translated
	{ "AsciiCircum", 0x5e }, // translated
	{ "Underscore", 0x5f }, // translated
	{ "QuoteLeft", 0x60 }, // translated
	{ "BraceLeft", 0x7b }, // translated
	{ "Bar", 0x7c },
	{ "BraceRight", 0x7d }, // translated
	{ "AsciiTilde", 0x7e }, // translated
	{ "nobreakspace", 0x0a0 },
	{ "exclamdown", 0x0a1 },
	{ "cent", 0x0a2 },
	{ "sterling", 0x0a3 },
	{ "currency", 0x0a4 },
	{ "yen", 0x0a5 },
	{ "brokenbar", 0x0a6 },
	{ "section", 0x0a7 },
	{ "diaeresis", 0x0a8 },
	{ "copyright", 0x0a9 },
	{ "ordfeminine", 0x0aa },
	{ "guillemotleft", 0x0ab },
	{ "notsign", 0x0ac },
	{ "hyphen", 0x0ad },
	{ "registered", 0x0ae },
	{ "macron", 0x0af },
	{ "degree", 0x0b0 },
	{ "plusminus", 0x0b1 },
	{ "twosuperior", 0x0b2 },
	{ "threesuperior", 0x0b3 },
	{ "acute", 0x0b4 },
	{ "mu", 0x0b5 },
	{ "paragraph", 0x0b6 },
	{ "periodcentered", 0x0b7 },
	{ "cedilla", 0x0b8 },
	{ "onesuperior", 0x0b9 },
	{ "masculine", 0x0ba },
	{ "guillemotright", 0x0bb },
	{ "onequarter", 0x0bc },
	{ "onehalf", 0x0bd },
	{ "threequarters", 0x0be },
	{ "questiondown", 0x0bf },
	{ "Agrave", 0x0c0 },
	{ "Aacute", 0x0c1 },
	{ "Acircumflex", 0x0c2 },
	{ "Atilde", 0x0c3 },
	{ "Adiaeresis", 0x0c4 },
	{ "Aring", 0x0c5 },
	{ "AE", 0x0c6 },
	{ "Ccedilla", 0x0c7 },
	{ "Egrave", 0x0c8 },
	{ "Eacute", 0x0c9 },
	{ "Ecircumflex", 0x0ca },
	{ "Ediaeresis", 0x0cb },
	{ "Igrave", 0x0cc },
	{ "Iacute", 0x0cd },
	{ "Icircumflex", 0x0ce },
	{ "Idiaeresis", 0x0cf },
	{ "ETH", 0x0d0 },
	{ "Ntilde", 0x0d1 },
	{ "Ograve", 0x0d2 },
	{ "Oacute", 0x0d3 },
	{ "Ocircumflex", 0x0d4 },
	{ "Otilde", 0x0d5 },
	{ "Odiaeresis", 0x0d6 },
	{ "multiply", 0x0d7 },
	{ "Ooblique", 0x0d8 },
	{ "Ugrave", 0x0d9 },
	{ "Uacute", 0x0da },
	{ "Ucircumflex", 0x0db },
	{ "Udiaeresis", 0x0dc },
	{ "Yacute", 0x0dd },
	{ "THORN", 0x0de },
	{ "ssharp", 0x0df },
	{ "agrave", 0x0e0 },
	{ "aacute", 0x0e1 },
	{ "acircumflex", 0x0e2 },
	{ "atilde", 0x0e3 },
	{ "adiaeresis", 0x0e4 },
	{ "aring", 0x0e5 },
	{ "ae", 0x0e6 },
	{ "ccedilla", 0x0e7 },
	{ "egrave", 0x0e8 },
	{ "eacute", 0x0e9 },
	{ "ecircumflex", 0x0ea },
	{ "ediaeresis", 0x0eb },
	{ "igrave", 0x0ec },
	{ "iacute", 0x0ed },
	{ "icircumflex", 0x0ee },
	{ "idiaeresis", 0x0ef },
	{ "eth", 0x0f0 },
	{ "ntilde", 0x0f1 },
	{ "ograve", 0x0f2 },
	{ "oacute", 0x0f3 },
	{ "ocircumflex", 0x0f4 },
	{ "otilde", 0x0f5 },
	{ "odiaeresis", 0x0f6 },
	{ "division", 0x0f7 },
	{ "oslash", 0x0f8 },
	{ "ugrave", 0x0f9 },
	{ "uacute", 0x0fa },
	{ "ucircumflex", 0x0fb },
	{ "udiaeresis", 0x0fc },
	{ "yacute", 0x0fd },
	{ "thorn", 0x0fe },
	{ "ydiaeresis", 0x0ff },
	{ "Back ", 0x1061 },
	{ "Forward ", 0x1062 },
	{ "Stop ", 0x1063 },
	{ "Refresh ", 0x1064 },
	{ "VolumeDown", 0x1070 },
	{ "VolumeMute ", 0x1071 },
	{ "VolumeUp", 0x1072 },
	{ "BassBoost", 0x1073 },
	{ "BassUp", 0x1074 },
	{ "BassDown", 0x1075 },
	{ "TrebleUp", 0x1076 },
	{ "TrebleDown", 0x1077 },
	{ "MediaPlay ", 0x1080 },
	{ "MediaStop ", 0x1081 },
	{ "MediaPrev ", 0x1082 },
	{ "MediaNext ", 0x1083 },
	{ "MediaRecord", 0x1084 },
	{ "HomePage ", 0x1090 },
	{ "Favorites ", 0x1091 },
	{ "Search ", 0x1092 },
	{ "Standby", 0x1093 },
	{ "OpenUrl", 0x1094 },
	{ "LaunchMail ", 0x10a0 },
	{ "LaunchMedia", 0x10a1 },
	{ "Launch0 ", 0x10a2 },
	{ "Launch1 ", 0x10a3 },
	{ "Launch2 ", 0x10a4 },
	{ "Launch3 ", 0x10a5 },
	{ "Launch4 ", 0x10a6 },
	{ "Launch5 ", 0x10a7 },
	{ "Launch6 ", 0x10a8 },
	{ "Launch7 ", 0x10a9 },
	{ "Launch8 ", 0x10aa },
	{ "Launch9 ", 0x10ab },
	{ "LaunchA ", 0x10ac },
	{ "LaunchB ", 0x10ad },
	{ "LaunchC ", 0x10ae },
	{ "LaunchD ", 0x10af },
	{ "LaunchE ", 0x10b0 },
	{ "LaunchF ", 0x10b1 },
	{ "MediaLast", 0x1fff },
	{ "unknown", 0xffff }
};
